Package: BayesianTools
Title: General-Purpose MCMC and SMC Samplers and Tools for Bayesian
        Statistics
Version: 0.1.9
Date: 2026-03-09
Authors@R: c(person("Florian", "Hartig", email = "florian.hartig@biologie.uni-regensburg.de", role = c("aut", "cre"), comment=c(ORCID="0000-0002-6255-9059")), person("Francesco", "Minunno", role = c("aut")), person("Stefan", " Paul", role = c("aut") ),  person("David", "Cameron", role = "ctb"), person("Tankred", "Ott", role = "ctb"), person("Maximilian", "Pichler", role = "ctb"))
Description: General-purpose MCMC and SMC samplers, as well as plots and
    diagnostic functions for Bayesian statistics, with a particular focus on
    calibrating complex system models. Implemented samplers include various
    Metropolis MCMC variants (including adaptive and/or delayed rejection MH), the
    T-walk, two differential evolution MCMCs, two DREAM MCMCs, and a sequential
    Monte Carlo (SMC) particle filter.
Depends: R (>= 3.1.2)
License: GPL-3
Imports: coda, emulator, mvtnorm, tmvtnorm, IDPmisc, Rcpp (>= 0.12.12),
        ellipse, numDeriv, msm, MASS, Matrix, stats, utils, graphics,
        DHARMa, gap, bridgesampling
Suggests: DEoptim, sensitivity, knitr, rmarkdown, roxygen2, testthat
LinkingTo: Rcpp
RoxygenNote: 7.3.3
URL: https://github.com/florianhartig/BayesianTools,
        https://florianhartig.github.io/BayesianTools/
BugReports: https://github.com/florianhartig/BayesianTools/issues
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-03-09 14:48:52 UTC; florian
Author: Florian Hartig [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6255-9059>),
  Francesco Minunno [aut],
  Stefan Paul [aut],
  David Cameron [ctb],
  Tankred Ott [ctb],
  Maximilian Pichler [ctb]
Maintainer: Florian Hartig <florian.hartig@biologie.uni-regensburg.de>
Repository: CRAN
Date/Publication: 2026-03-10 06:10:31 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 13:35:08 UTC; unix
Archs: BayesianTools.so.dSYM
