# Generated by roxygen2: do not edit by hand

S3method(getSample,MCMC)
S3method(getSample,MCMC_refClass)
S3method(getSample,data.frame)
S3method(getSample,double)
S3method(getSample,integer)
S3method(getSample,list)
S3method(getSample,matrix)
S3method(getSample,mcmc)
S3method(getSample,mcmc.list)
S3method(getSample,mcmcSampler)
S3method(getSample,mcmcSamplerList)
S3method(getSample,smcSampler)
S3method(getSample,smcSamplerList)
S3method(plot,mcmcSampler)
S3method(plot,mcmcSamplerList)
S3method(plot,smcSampler)
S3method(plot,smcSamplerList)
S3method(print,BayesianSetup)
S3method(print,mcmcSampler)
S3method(print,mcmcSamplerList)
S3method(print,prior)
S3method(print,proposalGenerator)
S3method(print,smcSampler)
S3method(print,smcSamplerList)
S3method(summary,mcmcSampler)
S3method(summary,mcmcSamplerList)
S3method(summary,smcSampler)
export(DE)
export(DEzs)
export(DIC)
export(DREAM)
export(DREAMzs)
export(GOF)
export(MAP)
export(Metropolis)
export(Twalk)
export(VSEM)
export(VSEMcreateLikelihood)
export(VSEMcreatePAR)
export(VSEMgetDefaults)
export(WAIC)
export(applySettingsDefault)
export(bridgesample)
export(calibrationTest)
export(checkBayesianSetup)
export(convertCoda)
export(correlationPlot)
export(createBayesianSetup)
export(createBetaPrior)
export(createLikelihood)
export(createMcmcSamplerList)
export(createMixWithDefaults)
export(createPosterior)
export(createPrior)
export(createPriorDensity)
export(createProposalGenerator)
export(createSmcSamplerList)
export(createTruncatedNormalPrior)
export(createUniformPrior)
export(gelmanDiagnostics)
export(generateParallelExecuter)
export(generateTestDensityMultiNormal)
export(getCredibleIntervals)
export(getPanels)
export(getPossibleSamplerTypes)
export(getPredictiveDistribution)
export(getPredictiveIntervals)
export(getSample)
export(getVolume)
export(likelihoodAR1)
export(likelihoodIidNormal)
export(marginalLikelihood)
export(marginalPlot)
export(mergeChains)
export(plotDiagnostic)
export(plotSensitivity)
export(plotTimeSeries)
export(plotTimeSeriesResiduals)
export(plotTimeSeriesResults)
export(runMCMC)
export(sampleMetropolis)
export(smcSampler)
export(stopParallel)
export(testDensityBanana)
export(testDensityInfinity)
export(testDensityMultiNormal)
export(testDensityNormal)
export(testLinearModel)
export(tracePlot)
export(updateProposalGenerator)
export(vsemC)
import(DHARMa)
import(Rcpp)
import(bridgesampling)
import(coda)
import(graphics)
import(stats)
importFrom(utils,flush.console)
importFrom(utils,methods)
importFrom(utils,modifyList)
useDynLib(BayesianTools, .registration = TRUE)
